<?php

namespace Lavalite\Demo\Http\Controllers;

use App\Http\Controllers\AdminController as AdminController;
use Form;
use Lavalite\Demo\Http\Requests\DemoAdminRequest;
use Lavalite\Demo\Interfaces\DemoRepositoryInterface;
use Lavalite\Demo\Models\Demo;

/**
 * Admin controller class.
 */
class DemoAdminController extends AdminController
{
    /**
     * Initialize demo controller.
     *
     * @param type DemoRepositoryInterface $demo
     *
     * @return type
     */
    public function __construct(DemoRepositoryInterface $demo)
    {
        parent::__construct();
        $this->repository = $demo;
    }

    /**
     * Display a list of demo.
     *
     * @return Response
     */
    public function index(DemoAdminRequest $request)
    {
        $demos  = $this->repository->setPresenter('\\Lavalite\\Demo\\Repositories\\Presenter\\DemoListPresenter')
                                            ->scopeQuery(function($query){
                                                return $query->orderBy('id','DESC');
                                            })->paginate();
        $this   ->theme->prependTitle(trans('demo::demo.names').' :: ');
        $view   = $this->theme->of('demo::admin.demo.index')->render();

        $this->responseCode = 200;
        $this->responseMessage = trans('messages.success.loaded', ['Module' => trans('demo::demo.name')]);
        $this->responseData = $demos['data'];
        $this->responseMeta = $demos['meta'];
        $this->responseView = $view;
        $this->responseRedirect = '';
        return $this->respond($request);
    }

    /**
     * Display demo.
     *
     * @param Request $request
     * @param int     $id
     *
     * @return Response
     */
    public function show(DemoAdminRequest $request, Demo $demo)
    {
        if (!$demo->exists) {
            $this->responseCode = 404;
            $this->responseMessage = trans('messages.success.notfound', ['Module' => trans('demo::demo.name')]);
            $this->responseView = view('demo::admin.demo.new');
            return $this -> respond($request);
        }

        Form::populate($demo);
        $this->responseCode = 200;
        $this->responseMessage = trans('messages.success.loaded', ['Module' => trans('demo::demo.name')]);
        $this->responseData = $demo;
        $this->responseView = view('demo::admin.demo.show', compact('demo'));
        return $this -> respond($request);
    }

    /**
     * Show the form for creating a new demo.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DemoAdminRequest $request)
    {

        $demo = $this->repository->newInstance([]);

        Form::populate($demo);

        $this->responseCode = 200;
        $this->responseMessage = trans('messages.success.loaded', ['Module' => trans('demo::demo.name')]);
        $this->responseData = $demo;
        $this->responseView = view('demo::admin.demo.create', compact('demo'));
        return $this -> respond($request);

    }

    /**
     * Create new demo.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DemoAdminRequest $request)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $demo = $this->repository->create($attributes);

            $this->responseCode = 201;
            $this->responseMessage = trans('messages.success.created', ['Module' => trans('demo::demo.name')]);
            $this->responseData = $demo;
            $this->responseMeta = '';
            $this->responseRedirect = trans_url('/admin/demo/demo/'.$demo->getRouteKey());
            $this->responseView = view('demo::admin.demo.create', compact('demo'));

            return $this -> respond($request);

        } catch (Exception $e) {
            $this->responseCode = 400;
            $this->responseMessage = $e->getMessage();
            return $this -> respond($request);
        }
    }

    /**
     * Show demo for editing.
     *
     * @param Request $request
     * @param int     $id
     *
     * @return Response
     */
    public function edit(DemoAdminRequest $request, Demo $demo)
    {
        Form::populate($demo);
        $this->responseCode = 200;
        $this->responseMessage = trans('messages.success.loaded', ['Module' => trans('demo::demo.name')]);
        $this->responseData = $demo;
        $this->responseView = view('demo::admin.demo.edit', compact('demo'));

        return $this -> respond($request);
    }

    /**
     * Update the demo.
     *
     * @param Request $request
     * @param int     $id
     *
     * @return Response
     */
    public function update(DemoAdminRequest $request, Demo $demo)
    {
        try {

            $attributes = $request->all();

            $demo->update($attributes);

            $this->responseCode = 204;
            $this->responseMessage = trans('messages.success.updated', ['Module' => trans('demo::demo.name')]);
            $this->responseData = $demo;
            $this->responseRedirect = trans_url('/admin/demo/demo/'.$demo->getRouteKey());

            return $this -> respond($request);

        } catch (Exception $e) {

            $this->responseCode = 400;
            $this->responseMessage = $e->getMessage();
            $this->responseRedirect = trans_url('/admin/demo/demo/'.$demo->getRouteKey());

            return $this -> respond($request);
        }
    }

    /**
     * Remove the demo.
     *
     * @param int $id
     *
     * @return Response
     */
    public function destroy(DemoAdminRequest $request, Demo $demo)
    {

        try {

            $t = $demo->delete();

            $this->responseCode = 202;
            $this->responseMessage = trans('messages.success.deleted', ['Module' => trans('demo::demo.name')]);
            $this->responseRedirect = trans_url('/admin/demo/demo/0');

            return $this -> respond($request);

        } catch (Exception $e) {

            $this->responseCode = 400;
            $this->responseMessage = $e->getMessage();
            $this->responseRedirect = trans_url('/admin/demo/demo/'.$demo->getRouteKey());

            return $this -> respond($request);

        }
    }
}
